/*

  SAMPMIX.C	MEX file corresponding to SAMPMIX.M

  function samp=sampmix(alpha,mun,vn,mup,vp);
  
*/

#include <math.h>
#include "mex.h"
#include "randn.c"


/* Input Arguments */

#define	ALPHA_IN	prhs[0]
#define	MUN_IN	  prhs[1]
#define	VN_IN	  prhs[2]
#define	MUP_IN	  prhs[3]
#define	VP_IN	  prhs[4]

/* Output Arguments */

#define	SAMP_OUT	plhs[0]

#define	max(A, B)	((A) > (B) ? (A) : (B))
#define	min(A, B)	((A) < (B) ? (A) : (B))
#define pi 3.14159265

static void sampmix(
	double	*samp,
	double	*alpha,
	double	*mun,
	double	*vn,
	double	*mup,
	double	*vp
	)
{
  double swit, spos, sneg, svn, svp;
  int done=0;

  svn=sqrt(vn[0]);
  svp=sqrt(vp[0]);

  while (!done) {
    swit=drand48();
    if (swit<=alpha[0]) /* sample from the negative side */
      {
	sneg=mun[0]+randn()*svn;
	if (sneg<0) 
	  {
	    *samp=sneg;
	    done=1;
	  }
      }
    else 
      {
	spos=mup[0]+randn()*svp;
	if (spos>0) 
	  {
	    *samp=spos;
	    done=1;
	  }
      }
  }
  return;
}

void mexFunction(
	int		nlhs,
	Matrix	*plhs[],
	int		nrhs,
	Matrix	*prhs[]
	)
{
	double	*samp, *alpha, *mun, *vn, *mup, *vp;
	unsigned int	m,n;

	/* Check for proper number of arguments */

	if (nrhs != 5) {
		mexErrMsgTxt("SAMPMIX requires five input arguments.");
	} else if (nlhs > 1) {
		mexErrMsgTxt("SAMPMIX requires one output argument.");
	}

	/* Check the dimensions of alpha. Alpha can be 1 X 1 only. */

	m = mxGetM(ALPHA_IN);
	n = mxGetN(ALPHA_IN);
	if (!mxIsNumeric(ALPHA_IN) || mxIsComplex(ALPHA_IN) || 
		!mxIsFull(ALPHA_IN)  || !mxIsDouble(ALPHA_IN) ||
		(max(m,n) != 1) || (min(m,n) != 1)) {
		mexErrMsgTxt("SAMPMIX requires that alpha be a 1 x 1 vector.");
	}

	/* Create a matrix for the return argument */

	SAMP_OUT = mxCreateFull(1, 1, REAL);

	/* Assign pointers to the various parameters */

	samp = mxGetPr(SAMP_OUT);
	alpha = mxGetPr(ALPHA_IN);
	mun = mxGetPr(MUN_IN);
	vn = mxGetPr(VN_IN);
	mup = mxGetPr(MUP_IN);
	vp = mxGetPr(VP_IN); 
	
	/* Do the actual computations in a subroutine */
	sampmix(samp,alpha,mun,vn,mup,vp);
	return;
}


