% function samp=samptail(mu,v);
%
% Sample from the positive tail of a Gaussian, mu has to be negative. 
% Will sample from the distribution of x given that x>0. 

function samp=samptail(mu,v);
tiny=exp(-600);

if mu>0,
  disp('mu must be negative');
  return;
end;

% form an exponential distribution bound by looking at the value and
% derivative at zero. 

lambda= -v/(mu+tiny);

pexp0=1/(lambda+tiny);                    % prob density of exponential at 0
pgauss0=1/sqrt(2*pi*v)*exp(-mu^2/(2*v));   % prob density of Gaussian at 0

done=0;
while ~done,
  % sample from the exp dist.
  
  y=-lambda*log(rand);
  
  % reject according to relative probabilities.
  
  pgaussy=1/sqrt(2*pi*v)*exp(-(y-mu)^2/(2*v));
  pexpy=1/lambda*exp(-y/lambda);
  
  if (pexpy*pgauss0)==0,
    accept=1;
  else
    accept=pgaussy*pexp0/(pexpy*pgauss0);
  end;

  if rand<accept,
    done=1;
  end;
end;

% fprintf('\n');

samp=y;

plt=0;
if plt,
  qq=0:0.01:2;
  qqa=1/sqrt(2*pi*v)*exp(-(qq-mu).^2/(2*v))*pexp0/(pgauss0+tiny);
  qqb=1/lambda*exp(-qq/lambda);
  
  clg;
  subplot(211);
  plot(qq,qqa,'r');
  hold on;
  plot(qq,qqb,'g');
  
  subplot(212);
  semilogy(qq,qqa,'r');
  hold on;
  semilogy(qq,qqb,'g');
end;

