/**************************************************************
 *
 *
 * $Header:$
 *
 * $Log:$
 *
 * MODULE DESCRIPTION
 *
 *   twogauss - MEx file to do sampling from
 *   a distribution made up of two halves of a
 *   gaussian
 *
 *
 * $Author:$
 *
 *************************************************************/

#include <math.h>
#include <stdlib.h>
#include "mex.h"
#include "threegauss.h"
#include "twogauss.h"

static  double  pi2 = 2.0 * M_PI;
static  double  tiny = 10 / HUGE;


double threegauss(
double		mun,
double		vn,
double		mup,
double		vp
)
{
  static  double  lpi2;
  double	pneg0, ppos0, pneg1, ppos1;
  double        alpha[3], asum,sw,P[3];
  double	zn, zp, zna, zpa, zp1,zs, stdn, stdp, r;
  double	intneg, intpos, inttot, sneg, spos;
  int		done = 0;
  double  samp;

  lpi2 = log(pi2);

  pneg0 = 1.0 / sqrt(pi2 * vn) * exp(-(mun * mun) / (2 * vn));
  ppos0 = 1.0 / sqrt(pi2 * vp) * exp(-(mup * mup) / (2 * vp));

  pneg1 = 1.0 / sqrt(pi2 * vn) * exp(-((mun-1.0) * (mun-1.0)) / (2 * vn));
  ppos1 = 1.0 / sqrt(pi2 * vp) * exp(-((mup-1.0) * (mup-1.0)) / (2 * vp));

  /* weights */
  alpha[0]=1.0;
  alpha[1]=pneg0/(ppos0+tiny)+tiny;
  alpha[2]=ppos1/(pneg1+tiny)*pneg0/(ppos0+tiny)+tiny; 
  asum=alpha[0]+alpha[1]+alpha[2];

  alpha[0]=alpha[0]/asum;
  alpha[1]=alpha[1]/asum;
  alpha[2]=alpha[2]/asum;

  /* sample from mixture once */
  sw=drand48();
  
  if (sw<alpha[0])
    {
      sneg=mun+randn()*sqrt(vn);
      if (sneg<=0.0)
	{
	  samp=sneg;
	  return samp;
	}
    }
  else if (sw<alpha[0]+alpha[1])
    {
      spos=mup+randn()*sqrt(vp);
      if (spos>0.0 && spos<1.0)
	{
	  samp=spos;
	  return samp;
	}
    }
  else
    {
      sneg=mun+randn()*sqrt(vn);
      if (sneg>1.0)
	{
	  samp=sneg;
	  return samp;
	}
    }

  P[0]=alpha[0]*cumnorm(mun,vn);
  P[1]=alpha[1]*(cumnorm(-mup,vp)-cumnorm(1-mup,vp));
  P[2]=alpha[2]*cumnorm(1-mun,vn);

  inttot=P[0]+P[1]+P[2];
  if (inttot==0)
    {
    samp=0;
    return samp;
    }

  P[0]/=inttot;   P[1]/=inttot;  P[2]/=inttot;

  zn = mun / sqrt(vn);
  zp = mup / sqrt(vp);
  zp1= (mup-1.0)/sqrt(vp);
  zs= (mun-1.0)/sqrt(vn);

  r=drand48();

  if ( r < P[0])
    if (zn > 1.0)
      samp=-samptail(-mun,vn);
    else
      samp=-samprej(-mun,vn);
  else if (r >=P[0] && r < P[0]+P[1])
    {
      done=0;
      while (0==done)
	{
	  if (zp<-1.0)
	    samp=samptail(mup,vp);
	  else if (zp1<1.0)
	    samp=samprej(mup,vp);
	  else 
	    samp=1-samptail(1-mup,vp);
	  if (samp>0.0 && samp<1.0)
	    done=1;
	}
    }
  else if  (r >= P[0]+P[1])
    {
      if (zs<-1.0)
	samp=1.0+samptail(mun-1.0,vn);
      else
	samp=1.0+samprej(mun-1.0,vn);
    }
  return samp;
}


